import React from 'react';
import axios from 'axios';
import {NSToast} from './NSToast';
import { scrollToTop } from '../lib/helpers';
import { isAuth } from '../lib/isAuth';
import getAPIUrl from '../lib/generateAPIUrl';

export class NSAddressMultiple extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            ...props,
            // Permet de sauvegarder l'adresse en cours de modification
            address : {
                civility             : '',
                lastname             : '',
                firstname            : '',
                companyName          : '',
                line1                : '',
                line2                : '',
                zipcode              : '',
                city                 : '',
                isoCountryCode       : '',
                phone                : '',
                phone_mobile         : '',
                idx_delivery_address : '',
                idx_billing_address  : ''
            },
            user           : { addresses: [] },
            editMode       : false,
            cart           : {},
            countries      : [],
            useSameAddress : false,
            index          : -1,
            lastCountry    : 'FR',
        };
    }
    /**
     * @returns {object} retourne le state de NSAddress
     */
    getParentState() {
        return this.state;
    }
    async componentDidMount() {
        try {
            const userAuth = await isAuth();
            if (!userAuth.isLogged) throw userAuth;
            const countries = await axios.get(`${getAPIUrl()}territory/countries`);
            this.setState({
                user        : userAuth.client,
                countries   : countries.data,
                lastCountry : userAuth.client.addresses[userAuth.client.delivery_address] ? userAuth.client.addresses[userAuth.client.delivery_address].isoCountryCode: countries.data[0].name
            });
        } catch (e) {
            console.error(e)
            NSToast.warn('addresses:error.notConnected', this.props.i18n);
            if (typeof window !== 'undefined') {
                window.location.pathname = window.location.pathname.indexOf('/cart/') > -1 ? '/cart/login' : '/login';
            }
        }
    }

    componentWillReceiveProps(props) {
        this.setState({ ...props });
    }
    /**
     * Fonction permettant de reset le forumlaire ainsi que this.state.address
     * @param {string} selecteurCssFormReset le selecteur css a passer afin de reset le formulaire
     * @return {void}
     */
    resetForm(selecteurCssFormReset) {
        const { address, user } = this.state;
        Object.keys(address).forEach(key => address[key] = '');
        this.setState({ editMode: true, address, index: user.addresses.length }, () => {
            document.querySelector(selecteurCssFormReset).reset();
        });
    }

    /**
     * Permet de mettre a jour l'etat en changeant l'address, l'index et le mode d'edition
     * @param {*} address adresse qui va être modifié, correspond aussi a user.addresses[index]
     * @param {*} index index de l'adresse a modifier dans le tableau user.addresses
     * @returns {void}
     */
    handleEditMode(address, index) {
        this.setState({ editMode: true, address, index });
    }
    /**
     * fonction indiquant si l'adresse est cochable ou non
     * Lors de l'édition d'une adresse, il est possible d'indiquer si cette adresse est de type facturation ou de type livraison
     * Si l'adresse est déja de type livraison ou facturation (ou les deux) alors il sera impossible de décocher la checkbox indiquant le type d'adresse (voir DNP AddressEdit)
     * @param {object} typeAddress adresse de livraison (user.delivery_address) ou de facturation (user.billing_address)
     * @param {object} addresses adresses de l'utilisateur (user.addresses)
     * @returns {boolean} si la checkbox est cochable ou non
     */
    addressDisabled(typeAddress, addresses)  {
        if ((Number(this.state.index) === Number(typeAddress)) || (addresses.length === 0)) {
            return true;
        }
        return false;
    }
    /**
     * Permet de créer une nouvelle adresse (ajout d'une adresse dans user.addresses)
     * @param {*} e event js
     * @returns {void}
     */
    async addAddress(e) {
        e.preventDefault();
        try {
            const { user, address } = this.state;
            const { idx_delivery_address, idx_billing_address } = address;
            delete address.idx_delivery_address;
            delete address.idx_billing_address;
            address.civility = Number(address.civility);
            if (address._id) {
                const idx = user.addresses.findIndex(adrs => adrs._id === address._id);
                if (idx >= 0) {
                    user.addresses[idx] = address;
                }
            } else {
                delete address._id;
                user.addresses.push(address);
            }
            const res = await axios.put(`${getAPIUrl()}v2/user/addresses`, {
                userId           : user._id,
                billing_address  : (idx_billing_address >= 0 && idx_billing_address !== '') ? Number(idx_billing_address) : Number(user.billing_address),
                delivery_address : (idx_delivery_address >= 0 && idx_delivery_address !== '') ? Number(idx_delivery_address) : Number(user.delivery_address),
                addresses        : user.addresses,
            });
            this.setState({
                user     : res.data,
                editMode : false,
            }, () => scrollToTop(500));
            NSToast.success('addresses:success.addressUpdateSuccess', this.props.i18n);
        } catch (error) {
            console.error(error);
            NSToast.error('addresses:error.addressUpdateFail', this.props.i18n);
        }
    }
    /**
     * Permet de mettre a jour le state du composant, a utiliser sur les inputs lors des onChange
     * @param {*} e event js
     * @returns {void}
     */
    handleAddressChange(e) {
        const {
            address, index
        } = this.state;
        if (e.target.name === 'idx_delivery_address' || e.target.name === 'idx_billing_address') {
            if (e.target.checked) {
                address[e.target.name] = index;
            } else {
                address[e.target.name] = '';
            }
        } else if (e.target.name === 'civility') {
            address[e.target.name] = Number(e.target.value);
        } else if ((e.target.type === 'checkbox' || e.target.type === 'radio') && e.target.name !== 'civility') {
            return;
        } else if (e.target.name === 'country') {
            address.isoCountryCode = e.target.value;
        } else {
            address[e.target.name] = e.target.value;
        }
        if (this.state.useSameAddress) {
            address[1] = address[0];
        }
        this.setState({ address });
    }

    /**
     * Lorsque l'adresse est modifié dans le tunnel de commande, cette fonction sera appelé
     * @param {*} e event js
     * @param {*} fn fonction appelé lorsque le processus d'enregistrement de l'adresse a reussi, cette fonction pourra par exemple être utilisé pour changer de page
     * @returns {void}
     */
    async submitAddresses(e, fn) {
        e.preventDefault();
        try {
            const { user } = this.state;
            if (user.addresses.length < 1) {
                return NSToast.error('addresses:error_msg.at_least_one_address', this.props.i18n);
            }
            try {
                const addresses = {
                    billing  : user.addresses[user.billing_address],
                    delivery : user.addresses[user.delivery_address]
                };
                const res = await axios.put(`${getAPIUrl()}v2/cart/addresses`, { cartId: window.localStorage.getItem('cart_id'), addresses });
                if (res.status === 200) {
                    fn();
                }
            } catch (error) {
                console.error(error);
                if (error.response && error.response.data.code === 'NOT_VALID_ADDRESS') {
                    return NSToast.error(error.response && error.response.data.message);
                }
                return NSToast.error('addresses:error_msg.fail_add_order_address', this.props.i18n);
            }
        } catch (error) {
            NSToast.error('addresses:error_msg.fail_add_order_address', this.props.i18n);
            console.error(error);
        }
    }

    fuseAddresses() {
        const { user } = this.state;
        user.addresses[1] = user.addresses[0];
        this.setState({ user });
    }
    /**
     * Permet de supprimer une adresse dans le tableau d'adresse (user.addresses)
     * @param {*} index index a supprimer dans le tableau d'adresse (user.addresses)
     * @returns {void}
     */
    async deleteAddress(index) {
        const { user } = this.state;
        let { billing_address, delivery_address } = user;
        if (index < billing_address) {
            billing_address--;
        }
        if (index < delivery_address) {
            delivery_address--;
        }
        user.addresses.splice(index, 1);
        const res = await axios.put(`${getAPIUrl()}v2/user/addresses`, {
            userId    : user._id,
            addresses : user.addresses,
            billing_address,
            delivery_address,
        });
        this.setState({ user: res.data });
    }
}
