import React from 'react';
import axios from 'axios';
import moment from 'moment';
import {NSToast} from './NSToast';
import getAPIUrl from '../lib/generateAPIUrl';
import statusColor from '../lib/statusColor';

/**
 * NSDetailOrder - Affiche le detail d'une commande
 * @return {React.Component}
 */
export class NSDetailOrder extends React.Component {
    constructor(props) {
        super(props);
        this.et_rate = 0.2;
        this.state = {
            order  : null,
            ticket : null,
        };
        this.cssHook = '';
        this.jsxHook = undefined;
    }
    componentDidMount = async () => {
        this.getOrder();
    };
    componentDidUpdate = async (prevProps, prevState) => {
        if (prevProps.numOrder == this.props.numOrder) return;
        this.getOrder();
    };
    getOrder = async () => {
        try {
            if (!this.props.numOrder) return;
            const PostBody = {
                filter   : { number: this.props.numOrder },
                populate : ['items.id'],
            };
            const order = await axios.post(`${getAPIUrl()}v2/order`, { PostBody });
            if (
                order.data.status !== 'FINISHED' &&
                order.data.status !== 'PAID'
            ) {
                return this.setState({ order: order.data });
            }
            const ticket = await this.getTicketValidity(order.data._id);
            return this.setState({ order: order.data, ticket });
        } catch (error) {
            return NSToast.error('account:order.error.getOrderError', this.props.i18n);
        }
    };

    render() {
        const { order, ticket } = this.state;
        if (!order) return '';
        const {
            createdAt, number, items, status, priceTotal,
        } = order;
        return (
            <div className="wrapDetailOrder">
                <h3>Commande N°{number}</h3>
                <h6>
                    Du {moment(new Date(createdAt)).format('lll')} -{' '}
                    {statusColor(status)}
                </h6>

                <table>
                    <thead>
                        <tr>
                            <th>REF</th>
                            <th>DESCRIPTION PRODUIT</th>
                            <th>QUANTITE</th>
                            <th>PU TTC</th>
                            <th>TOTAL TTC</th>
                        </tr>
                    </thead>
                    <tbody>
                        {items.map((item, i) => {
                            const { quantity, price, id } = item;
                            return (
                                <tr key={i}>
                                    <td>
                                        <p>{id.id}</p>
                                        <img
                                            src={`/${id.imageUrl}`}
                                            width="90"
                                            height="50"
                                            alt={id.name}
                                        />
                                    </td>
                                    <td>{id.details.description1.text}</td>
                                    <td>{quantity}</td>
                                    <td>
                                        {quantity >= 5
                                            ? price.unit.ati.aqlRound(2) /* price.special.ati.aqlRound(2) */
                                            : price.unit.ati.aqlRound(2)}{' '}
                                    €
                                    </td>
                                    <td>{price.total.ati.aqlRound(2)} €</td>
                                </tr>
                            );
                        })}
                    </tbody>
                </table>
                <div className="wrapperFooter">
                    <div>{this.showInfos()}</div>
                    <div>
                        <p className="ttc">
                            Total TTC: {priceTotal.ati.aqlRound(2)} €
                        </p>
                        <p className="tva">
                            Dont TVA:{' '}
                            {(priceTotal.ati * this.et_rate).aqlRound(2)} €
                        </p>
                    </div>
                </div>
                <style jsx="true">{`
                    h3 {
                        color: #fdbc58;
                    }
                    .btnYellow {
                        color: #fdbc58;
                    }
                    .wrapDetailOrder {
                        width: 100%;
                    }
                    .wrapperFooter {
                        display: flex;
                        justify-content: space-between;
                    }
                    table {
                        background: #fdbc58;
                        width: 100%;
                        color: black;
                        margin-top: 75px;
                    }
                    table td {
                        width: 20%;
                    }
                    thead tr {
                        height: 40px;
                        border-bottom: 2px solid black;
                    }
                    tbody tr {
                        height: 40px;
                        border-bottom: 1px solid black;
                        transition: 0.3s;
                    }
                    .footer {
                        display: flex;
                        flex-direction: row;
                        justify-content: space-around;
                    }
                    .ttc {
                        color: #fdbc58;
                        font-size: 22px;
                        text-align: end;
                    }
                    .tva {
                        text-align: end;
                    }
                    @media screen and (max-width: 768px) {
                        .wrapDetailOrder {
                            width: 100%;
                        }
                    }
                    ${this.cssHook};
                `}</style>
                {this.jsxHook}
            </div>
        );
    }
}
