import React from 'react';
import axios from 'axios';
import PropTypes from 'prop-types';
import { NSToast } from './NSToast';
import getAPIUrl from '../lib/generateAPIUrl';
import { getLangPrefix } from '../lib/helpers';

/**
 * NSLogin - Affiche le formulaire de connexion puis envoie la requete au serveur et stock le token en cas de succes
 * @prop none
 * @ - login: (object) contient deux couples clé/valeur, email & password
 * State - passwordForgotten: (bool) s'il est a "true", affiche le formulaire de reinitialisation de mot de passe
 * State - disableBtnNewPassword: (bool) s'il est a "true", desactive le bouton de pour "setter" un nouveau mot de passe
 *
 * @return {React.Component}
 */
export class NSLogin extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            login : {
                email    : '',
                password : '',
            },
            passwordForgotten     : false,
            disableBtnNewPassword : false,
        };
    }


    static contextTypes = {
        getPageState : PropTypes.func,
        onLangChange : PropTypes.func,
    };

    handleLoginSubmit(event, Router) {
        event.preventDefault();
        const email = document.querySelector('#email_login').value;
        const password = document.querySelector('#password_login').value;
        this.loginRequest(email, password, Router);
    }

    loginRequest = async (email, password, Router) => {
        try {
            const jwtToken = await axios.post(`${getAPIUrl()}v2/auth/login`, { username: email, password });
            if (jwtToken === undefined) return;
            axios.defaults.headers.common.Authorization = jwtToken.data.data;
            // Si on vient du panier on continue vers la facturation
            const { i18n, nsLanguages } = this.context.getPageState();
            let path = 'account';
            if (window.location.pathname.indexOf('cart') > -1) {
                path = 'cart/address';
            }

            if (Router) {
                Router.pushRoute(`${getLangPrefix(nsLanguages, i18n.language)}/${path}`);
            } else {
                window.location.pathname = `${getLangPrefix(nsLanguages, i18n.language)}/${path}`;
            }
        } catch (err) {
            console.error(err);
            if (err.response.data.code === 'LOGIN_DATA_ERROR') {
                NSToast.error('login:error.invalidCredential', this.props.i18n);
            }
        }
    };

    registerRequest = async (user, onSuccess, onError) => {
        try {
            await axios.put(`${getAPIUrl()}v2/user`, user);
            const jwtToken = await axios.post(`${getAPIUrl()}v2/auth/login`, {
                username : user.email,
                password : user.password,
            });
            axios.defaults.headers.common.Authorization = jwtToken.data.data;
            if (onSuccess) onSuccess(jwtToken.data.data);
        } catch (e) {
            if (onError) onError(e);
        }
    };

    async componentDidMount() {
        try {
            const userAuth = await isAuth();
            if (!userAuth.isLogged) throw userAuth;

            const { i18n, nsLanguages } = this.context.getPageState();
            if (window.location.pathname.indexOf('cart') > -1) { window.location.pathname = `${getLangPrefix(nsLanguages, i18n.language)}/cart/address`; } else { window.location.pathname = `${getLangPrefix(nsLanguages, i18n.language)}/account`; }
        } catch (e) {
            if (window.location.pathname.indexOf('cart') > -1) { NSToast.error('delivery:not_logged_payment', this.props.i18n); }
        }
    }

    async handleResetSubmit(event) {
        try {
            event.preventDefault();
            const result = await axios.post(`${getAPIUrl()}v2/user/resetpassword/${this.props.i18n.language}`, {
                email : this.state.login.email,
            });
            NSToast.success('login:valid_msg.resetPassword', this.props.i18n);
            setTimeout(() => this.setState({ passwordForgotten: false }), 3000);
            this.setState({ disableBtnNewPassword: true });
        } catch (err) {
            if (err.response) NSToast.error(err.response.data.translation[this.props.i18n.language]);
            else NSToast.error('common:error_occured', this.props.i18n);
        }
    }

    render() {
        const { passwordForgotten, disableBtnNewPassword } = this.state;
        return (
            <div className="client">
                {!passwordForgotten ? (
                    <div>
                        <div className="title">
                            DEJA CLIENT ? <br />
                            CONNECTEZ-VOUS !
                        </div>
                        <form
                            style={{ marginTop: '50px' }}
                            onSubmit={e => this.handleLoginSubmit(e)}
                        >
                            <div className="form-group row vcenter fieldsClient">
                                <label
                                    htmlFor="mailClient"
                                    className="col-md-3 col-form-label requireField"
                                >
                                    Email<span className="requireField">*</span>
                                </label>
                                <div className="col-md-9">
                                    <input
                                        type="email"
                                        className="form-control"
                                        name="email"
                                        id="mailClient"
                                        value={this.state.login.email}
                                        onChange={this.handleLoginChange}
                                        required
                                    />
                                </div>
                            </div>
                            <div className="form-group row vcenter fieldsClient">
                                <label
                                    htmlFor="passwordClient"
                                    className="col-md-3 col-form-label requireField"
                                >
                                    Mot de passe<span className="requireField">
                                        *
                                    </span>
                                </label>
                                <div className="col-md-9">
                                    <input
                                        type="password"
                                        className="form-control"
                                        name="password"
                                        id="passwordClient"
                                        value={this.state.login.password}
                                        onChange={this.handleLoginChange}
                                        required
                                    />
                                </div>
                            </div>
                            <div className="form-group row">
                                <div className="col-md-5 offset-md-3">
                                    <button type="submit">SE CONNECTER</button>
                                </div>
                                <div className="col-md-9 offset-md-3">
                                    <p>
                                        <a
                                            href=""
                                            onClick={
                                                this.togglePasswordForgotten
                                            }
                                            className="recupPass"
                                        >
                                            Mot de passe oublié ?
                                        </a>
                                    </p>
                                </div>
                            </div>
                            {/* <div className="fieldsClient">
                                    <div className="labels">
                                <label htmlFor="mailClient">Adresse mail<span className="requireField">*</span></label>
                                <label htmlFor="passwordClient">Mot de passe<span className="requireField">*</span></label>
                                    </div>
                                    <div className="inputs">
                                <input id="mailClient" type="email" name="email" value={email} onChange={this.handleLoginChange} required />
                                <input id="passwordClient" type="password" name="password" value={password}  />
                                <button type="submit">SE CONNECTER</button>
                                <a href="" onClick={this.togglePasswordForgotten} className="recupPass">Mot de passe oublié ?</a>
                                    </div>
                            </div> */}
                        </form>
                    </div>
                ) : (
                    <div>
                        <div className="title">
                            MOT DE PASSE OUBLIÉ ? <br />
                            RÉCUPÉREZ LE !
                        </div>
                        <form
                            style={{ marginTop: '50px' }}
                            onSubmit={e => this.handleResetSubmit(e)}
                        >
                            <div className=" form-group row fieldsClient vcenter">
                                <label
                                    className="col-sm-3"
                                    htmlFor="mailClient"
                                >
                                    Adresse mail<span className="requireField">
                                        *
                                    </span>
                                </label>
                                <input
                                    className="form-control col-sm-9"
                                    id="mailClient"
                                    type="email"
                                    name="email"
                                    value={this.state.login.email}
                                    onChange={this.handleLoginChange}
                                    required
                                />
                            </div>
                            <div className="row">
                                <div className="col-sm-9 offset-sm-3 form-buttons">
                                    <button
                                        type="submit"
                                        id="resetPass"
                                        disabled={disableBtnNewPassword}
                                    >
                                        ENVOYER
                                    </button>
                                </div>
                                <div className="col-sm-9 offset-sm-3 form-buttons">
                                    <a
                                        href=""
                                        onClick={this.togglePasswordForgotten}
                                        className="recupPass"
                                    >
                                        Retour à la page de connexion
                                    </a>
                                </div>
                            </div>
                        </form>
                    </div>
                )}
                <style jsx="true">{`
                    .client {
                        color: white;
                    }
                    .client .title {
                        font-size: 30px;
                        color: #fdbc58;
                        text-align: center;
                    }
                    .requireField {
                        color: #fff;
                        padding: 0;
                    }
                    button {
                        color: white;
                        background: rgb(253, 188, 88);
                        font-size: 22px;
                        border: none;
                        cursor: pointer;
                        text-align: center;
                        line-height: 35px;
                        width: fit-content;
                        margin-top: 8px;
                        transition: 0.4s;
                        padding-right: 10px;
                        padding-left: 10px;
                    }
                    button:hover {
                        background: rgb(247, 174, 63);
                    }
                    .recupPass {
                        margin-top: 10px;
                        font-size: 12px;
                        color: white;
                    }
                    .client {
                        display: flex;
                        flex-direction: column;
                        width: 100%;
                        padding-top: 30px;
                    }
                    .fieldsClient {
                        display: flex;
                        justify-content: center;
                        flex-direction: row;
                        margin: 0;
                    }
                    label span {
                        vertical-align: middle;
                        color: #fdbc58;
                    }
                    .requiredLabel {
                        font-size: 13px;
                        color: #fdbc58;
                        letter-spacing: 0;
                        text-align: right;
                        line-height: 21px;
                    }
                    button:disabled,
                    button[disabled] {
                        background: #cacaca;
                    }
                    .vcenter {
                        display: flex;
                        align-items: center;
                    }
                    form input.form-control {
                        background-color: black;
                        border: 1px solid white;
                        width: 100%;
                        height: 40px;
                        margin-top: 8px;
                        margin-bottom: 8px;
                        color: white;
                        padding-left: 10px;
                        border-radius: 0;
                    }
                    .form-buttons {
                        padding: 0;
                    }
                    @media screen and (max-width: 990px) {
                        .client {
                            margin-top: 100px;
                        }
                        button,
                        .recupPass {
                            align-self: center;
                        }
                        div form div .labels {
                            width: 100%;
                            text-align: center;
                        }
                        div form div .inputs {
                            width: 100%;
                            text-align: center;
                        }
                        label {
                            text-align: center;
                        }
                    }
                `}</style>
            </div>
        );
    }
}
