import React from 'react';
import axios from 'axios';
import moment from 'moment';
import {NSToast} from './NSToast';
import getAPIUrl from '../lib/generateAPIUrl';
import statusColor from '../lib/statusColor';

/**
 * NSOrderList description
 * @return {React.Component}
 */
export class NSOrderList extends React.Component {
    constructor(props) {
        super(props);
        let { page } = this.props;
        const LIMIT = 10;
        if (!page || page < 1) page = 1;
        const min = (page - 1) * LIMIT;
        const max = LIMIT * page;
        this.state = {
            page      : Number(page) ? Number(page) : 1,
            allOrders : null,
            orders    : null,
            limit     : LIMIT,
            count     : null,
            client    : null,
        };
        this.cssHook = '';
        this.jsxHook = undefined;
    }
    getOrders = async () => {
        try {
            const { page, limit } = this.state;
            const result = await axios.get(`${getAPIUrl()}orders/user/${
                this.props.client._id
            }?sort_creationDate=-1`);
            const { min, max } = this.getPagination(page, limit);
            this.setState({
                allOrders : result.data,
                orders    : result.data.slice(min, max),
                count     : result.data.length,
            });
        } catch (error) {
            console.error(error);
            return NSToast.error('account:orders.error.getOrders', this.props.i18n);
        }
    };
    componentDidMount = () => {
        if (!this.props || !this.props.client) return;
        this.getOrders();
    };
    componentDidUpdate = async (prevProps, prevState) => {
        if (!this.props || !this.props.client) return;
        if (prevProps.client && prevProps.client._id === this.props.client._id) {
            return;
        }
        this.getOrders();
    };
    getPagination = (page, limit) => {
        const min = (page - 1) * limit;
        const max = limit * page;
        return { min, max };
    };
    onChangePage = (nextOrPrev) => {
        let {
            page, limit, orders, allOrders,
        } = this.state;
        page = Number(page);
        nextOrPrev == '-' ? (page -= 1) : (page += 1);
        if (nextOrPrev == '-' && page == 0) return;
        if (
            nextOrPrev == '+' &&
            (limit > orders.length ||
                allOrders[allOrders.length - 1]._id ==
                orders[orders.length - 1]._id)
        ) {
            return;
        }
        if (!page || page < 1) page = 1;
        const { min, max } = this.getPagination(page, limit);
        window.location.pathname = `/account/orders/${page}`;
        this.setState({
            page,
            orders : allOrders.slice(min, max),
        });
    };
    render() {
        const { page, orders } = this.state;
        if (!orders) return '';
        if (!orders.length) {
            return (
                <div className="wrapOrder">
                    <h1>Aucune commande</h1>
                </div>
            );
        }
        return (
            <div className="wrapOrder">
                <h3>MES COMMANDES</h3>
                <div className="wrapTable">
                    <table>
                        <thead>
                            <tr>
                                <th>REF</th>
                                <th>DATE</th>
                                <th>MONTANT TTC</th>
                                <th>STATUS</th>
                            </tr>
                        </thead>
                        <tbody>
                            {orders.map((order, idx) => {
                                const {
                                    number,
                                    createdAt,
                                    priceTotal,
                                    status,
                                } = order;
                                return (
                                    <a
                                        key={number}
                                        href={`/account/orders/details/${number}`}
                                    >
                                        <tr>
                                            <td>{number}</td>
                                            <td>
                                                {moment(new Date(createdAt)).format('lll')}
                                            </td>
                                            <td>
                                                {priceTotal.ati.aqlRound(2)} €
                                            </td>
                                            <td>{statusColor(status)}</td>
                                        </tr>
                                    </a>
                                );
                            })}
                        </tbody>
                    </table>
                    <div className="footer">
                        <button onClick={() => this.onChangePage('-')}>
                            {'<'}
                        </button>
                        <div className="numPage">{page}</div>
                        <button onClick={() => this.onChangePage('+')}>
                            {'>'}
                        </button>
                    </div>
                </div>

                <style jsx="true">{`
                    h3 {
                        color: #fdbc58;
                    }
                    .wrapOrder {
                        max-width: 900px;
                        margin: 0 auto;
                        font-family: 'Oswald', sans-serif;
                        display: flex;
                        justify-content: center;
                        width: 100%;
                        margin-bottom: 50px;
                        flex-direction: column;
                    }
                    .wrapTable {
                        min-height: 596px;
                        background: #fdbc58;
                        justify-content: space-between;
                        display: flex;
                        margin-top: 92px;
                        flex-direction: column;
                    }
                    table {
                        background: #fdbc58;
                        width: 100%;
                        color: black;
                    }
                    table td {
                        width: 20%;
                    }
                    thead tr {
                        height: 50px;
                        border-bottom: 2px solid black;
                    }
                    tbody tr {
                        height: 50px;
                        border-bottom: 1px solid black;
                        cursor: pointer;
                        transition: 0.3s;
                    }
                    tbody tr:hover {
                        background: black;
                        color: white;
                    }
                    .footer {
                        display: flex;
                        flex-direction: row;
                        justify-content: space-around;
                    }
                    .numPage {
                        display: flex;
                        justify-content: center;
                        flex-direction: column;
                        width: 65px;
                        border-top-left-radius: 7px;
                        border-top-right-radius: 7px;
                        background: black;
                    }
                    button {
                        border: none;
                        border-top-left-radius: 7px;
                        border-top-right-radius: 7px;
                        color: white;
                        background: black;
                        font-size: 22px;
                        cursor: pointer;
                        text-align: center;
                        line-height: 35px;
                        width: 50px;
                        margin-top: 8px;
                        transition: 0.4s;
                        padding-right: 10px;
                        padding-left: 10px;
                    }
                    button:hover {
                        color: #fdbc58;
                    }
                    @media screen and (max-width: 1450px) {
                        .wrapOrder {
                            width: 90%;
                        }
                    }
                    ${this.cssHook};
                `}</style>
                {this.jsxHook}
            </div>
        );
    }
}
