import React from 'react';
import axios from 'axios';
import componentLoader from './componentLoader';
import getAPIUrl from './generateAPIUrl';

export const last = array => array[array.length - 1];

/**
 * Permet de récupérer le prefix de la langue courante
 * @param {*} nsLanguages tableau de langue récupéré en bdd
 * @param {*} currentLanguage langue courante (i18n.language)
 * @return {string} chaine vide aucune language par defaut trouvé, /en ou /fr etc. Si le currentLanguage n'est pas le language par defaut
 */
export function getLangPrefix(nsLanguages, currentLanguage) {
    const oDefaultLang = nsLanguages.find(language => language.defaultLanguage === true);
    let prefix = '';
    if (!oDefaultLang) {
        return prefix;
    }
    let defaultLang = '';
    defaultLang = oDefaultLang.code;
    // Si le language par defaut est différent du currentLanguage alors :/lang (ex: /fr) sinon on retourne une chaine vide
    if (defaultLang !== currentLanguage) {
        prefix = `/${currentLanguage}`;
    }
    return prefix;
}


export function hexToRGBA(hex, alpha) {
    hex = (`${hex}`).trim().replace(/#/g, ''); // trim and remove any leading # if there (supports number values as well)
    if (!/^(?:[0-9a-fA-F]{3}){1,2}$/.test(hex)) throw ('not a valid hex string'); // Regex Validator
    if (hex.length == 3) { hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2]; } // support short form
    const b_int = parseInt(hex, 16);
    return `rgba(${[
        (b_int >> 16) & 255, // R
        (b_int >> 8) & 255, // G
        b_int & 255, // B
        alpha || 1,  // add alpha if is set
    ].join(',')})`;
}


export function shadeColor(color, percent) {
    const f = parseInt(color.slice(1), 16);
    const t = percent < 0 ? 0 : 255;
    const p = percent < 0 ? percent * -1 : percent;
    const R = f >> 16;
    const G = f >> 8 & 0x00FF;
    const B = f & 0x0000FF;
    return `#${(0x1000000 + (Math.round((t - R) * p) + R) * 0x10000 + (Math.round((t - G) * p) + G) * 0x100 + (Math.round((t - B) * p) + B)).toString(16).slice(1)}`;
}

export function simplifyPath(path) {
    return path.replace(/(\?.*)$/, '').split('/');
}

export const initPage = async (req = undefined, lang) => {
    // on recupére le block cms ayant pour id "header", un block cms peut contenir plusieurs composant
    const cmsHeader = await axios.post(`${getAPIUrl(req)}v2/component/ns-cms/header`, { lang: req.query && req.query.lang !== undefined ? req.query.lang : lang });
    if (cmsHeader.err || cmsHeader.data === null || cmsHeader.data === '') {
        console.error('\x1b[31m%s\x1b[0m', `Error: {pageStatic: ${_slug}}`);
    }
    // On charge les datas des composants contenu dans le BlocCMS si besoin
    const datasComponents = await componentLoader(cmsHeader.data, {}, getAPIUrl(req), req.query && req.query.lang !== undefined ? req.query.lang : lang);
    // Nous n'avons pas de data a charger pour ce BlocCMS, nous n'utilisons donc pas componentLoader
    const cmsFooter = await axios.post(`${getAPIUrl(req)}v2/component/ns-cms/footer`, { lang: req.query && req.query.lang !== undefined ? req.query.lang : lang });
    if (cmsFooter.err || cmsFooter.data === null || cmsFooter.data === '') {
        console.error('\x1b[31m%s\x1b[0m', `Error: {pageStatic: ${_slug}}`);
    }
    const breadcrumb = await axios.post(`${getAPIUrl()}v2/getBreadcrumb`, {
        url : simplifyPath(req.asPath),
    });
    return {
        oCmsHeader : cmsHeader.data,
        oCmsFooter : cmsFooter.data,
        breadcrumb : breadcrumb.data,
        ...datasComponents,
    };
};

export const isServer = () => typeof window === 'undefined';

export const scrollToTop = (to) => {
    window.scrollTo(to, 0);
};

export const getProductUrl = (product, lang) => {
    if (product === undefined) throw 'getProductUrl(product, lang) => /!\\ product is not defined';
    if (lang === undefined) throw 'getProductUrl(product, lang) => /!\\ lang is not defined';
    if (product.canonical && product.canonical !== '') {
        const slugs = product.canonical.split('/').slice(1, product.canonical.length - 1);
        if (slugs[0].length === 2) {
            return `/${slugs.slice(1).join('/')}`;
        }
        return `/${slugs.join('/')}`;
    }
    return `/${['other', product.slug[lang]].join('/')}`;
};

export const has =  (obj, key) => key.split('.').every((x) => {
    if (typeof obj !== 'object' || obj === null || !(x in obj)) { return false; }
    if (obj.constructor === Array) { obj = obj[0]; }
    obj = obj[x];
    return true;
});

export const base64ToString =  str => Buffer.from(str, 'base64').toString('binary');

export const stringToBase64 =  str => Buffer.from(str.toString(), 'binary');

export function getUrlParams(search) {
    const hashes = search.slice(search.indexOf('?') + 1).split('&');
    return hashes.reduce((params, hash) => {
        const [key, val] = hash.split('=');
        return Object.assign(params, { [key]: decodeURIComponent(val) });
    }, {});
}

/*
 * Fusionne 2 objets de manière récursive
 */
export function mergeRecursive(obj1, obj2) {
    for (var p in obj2) {
        try {
            if (obj2[p].constructor === Object) {
                obj1[p] = mergeRecursive(obj1[p], obj2[p]);
            } else {
                obj1[p] = obj2[p];
            }
        } catch(e) {
            obj1[p] = obj2[p];
        }
    }
    return obj1;
}