import React from 'react';
import axios from 'axios';
import PropTypes from 'prop-types';
import Pagination from 'rc-pagination';
import componentLoader from '../lib/componentLoader';
import getAPIUrl from '../lib/generateAPIUrl';
import {
    simplifyPath, initPage, last, base64ToString,
} from '../lib/helpers';
import 'rc-pagination/assets/index.css';
import { NSFilters } from '../components/NSFilters';
import { NSProductCard } from '../components/NSProductCard';
import { NSBreadcrumb } from '../components/NSBreadcrumb';
import jwtManager from '../lib/jwtManager';

const defaultItemPerPage = 15;

export class NSCategory extends React.Component {
    static getInitialProps = async function (req) {
        // Gestion de la langue et du slug
        const langs = await axios.post(`${getAPIUrl(req)}v2/languages`, { PostBody: { limit: 99 } });
        const lang  = req.query.lang || langs.data.datas.find(l => l.defaultLanguage === true).code;

        const slugs = req.query._slug.split('/');
        let page = 1;
        let slug = '';

        // Gestion des page dans l URL et du slug
        if (last(slugs) * 1 == last(slugs)) {
            page = last(slugs);
            slug = slugs[slugs.length - 2];
            slugs.pop();
        } else {
            page = 1;
            slug = last(slugs);
        }
        let baseUrl = '';
        if (req.req) {
            baseUrl = `${req.req.protocol}://${req.req.headers.host}${req.query.lang ? `/${req.query.lang}` : ''}/c/${slugs.join('/')}`;
        } else {
            baseUrl = `${window.location.origin}${req.query.lang ? `/${req.query.lang}` : ''}/c/${slugs.join('/')}`;
        }
        try {
            // On charge la page static pouvant contenir plusieurs BlocCMS comportant eu même plusieurs composant a charger
            let isEmpty = false;
            let notFound = false;
            const datasComponentsStatic = null;
            let datasComponentsCategory = null;
            // on récupère l'information pour voir les prix en HT ou en TTC
            const taxDisplay = this.props.taxDisplay;
            // on recupére le block cms ayant pour id "header", un block cms peut contenir plusieurs composant
            const initData = await initPage(req, lang);
            // On envoie une requete a l api category afin de recupérer la categorie souhaitée
            const promises = [];
            const categories = [];
            for (let i = 0; i < slugs.length; i++) {
                promises.push(axios.post(`${getAPIUrl(req)}v2/category`, {
                    lang,
                    PostBody : {
                        filter    : { [`translation.${lang}.slug`]: slugs[i] },
                        structure : {
                            displayOrder    : 1,
                            filters         : 1,
                            children        : 1,
                            productsList    : 1,
                            colorName       : 1,
                            metaDescription : 1,
                        },
                        populate : ['children'],
                    },
                    withFilter : true,
                }));
            }
            const res = await axios.all(promises);

            for (let i = 0; i < res.length; i++) {
                categories.push(res[i].data);
            }
            const category = categories[categories.length - 1];
            if (category.extraText && category.extraText !== '') {
                datasComponentsCategory = {
                    nsCms_extraText : { content: category.extraText },
                };
                datasComponentsCategory = await componentLoader(
                    { content: category.extraText },
                    datasComponentsCategory,
                    getAPIUrl(req),
                    req.query.lang || lang,
                );
            }
            let productsList = {
                data : {
                    datas    : [], priceMin : 0, priceMax : 0, count    : 0,
                },
            };
            if (category) {
                if (typeof window !== 'undefined' && window.localStorage.getItem('filters') && JSON.parse(window.localStorage.getItem('filters')).category !== category.name)
                    window.localStorage.removeItem('filters');
                productsList = await axios.post(`${getAPIUrl(req)}v2/products/category/${category._id}`, {
                    lang,
                    withStocks : true,
                    PostBody   : {
                        filter    : typeof window !== 'undefined' && window.localStorage.getItem('filters') && category.name === JSON.parse(window.localStorage.getItem('filters')).category ? JSON.parse(window.localStorage.getItem('filters')).filters : {},
                        structure : {
                            slug              : 1,
                            canonical         : 1,
                            visible           : 1,
                            weight            : 1,
                            pos               : 1,
                            stock             : 1,
                            reviews           : 1,
                            universe          : 1,
                            price             : 1,
                            type              : 1,
                            translation       : 1,
                            images            : 1,
                            pictos            : 1
                        },
                        limit : defaultItemPerPage,
                        page  : typeof window !== 'undefined' && window.localStorage.getItem('page') && category.name === JSON.parse(window.localStorage.getItem('page')).location ? JSON.parse(window.localStorage.getItem('page')).page : page,
                        sort  : { sortWeight: -1 },
                    },
                });
            } else {
                isEmpty = true;
                notFound = true;
            }
            const breadcrumb = await axios.post(
                `${getAPIUrl(req)}v2/getBreadcrumb`,
                {
                    url : simplifyPath(req.asPath),
                },
            );

            return {
                isEmpty,
                notFound,
                ...datasComponentsCategory,
                ...datasComponentsStatic,
                ...initData,
                url          : { asPath: req.asPath },
                query        : req.query,
                category,
                filtersData  : category.filters,
                productsList : productsList.data.datas,
                filters      : {
                    $and : [{
                        $or : [
                            {
                                [`price.${taxDisplay}.normal`] : {
                                    $gte : productsList.data.priceMin[taxDisplay] || 0,
                                    $lte : Math.ceil(productsList.data.priceMax[taxDisplay]) || 0,
                                },
                            }, {
                                [`price.${taxDisplay}.special`] : {
                                    $gte : productsList.data.priceMin[taxDisplay] || 0,
                                    $lte : Math.ceil(productsList.data.priceMax[taxDisplay]) || 0,
                                },
                            }],
                    }],
                },
                count       : productsList.data.count,
                min         : productsList.data.priceMin[taxDisplay],
                max         : productsList.data.priceMax[taxDisplay],
                current     : typeof window !== 'undefined' && window.localStorage.getItem('filters') && category.name === JSON.parse(window.localStorage.getItem('filters')).category ? JSON.parse(window.localStorage.getItem('filters')).page : page,
                baseUrl,
                categories,
                nsLanguages : langs.data,
                breadcrumb  : breadcrumb.data,
                lang,
                multilangue : langs.length > 0,
            };
        } catch (error) {
            console.error(error);
            return error;
        }
    };

    constructor(props) {
        super(props);
        this.state = {
            ...props,
            itemsPerPages : defaultItemPerPage,
        };
    }

    componentDidMount() {
        if (typeof window !== 'undefined' && window.localStorage.getItem('filters') && JSON.parse(window.localStorage.getItem('filters')).category !== category.name)
            window.localStorage.removeItem('filters');
    }

    onPageChange = (page, pageSize, e = undefined, filters = {}) => {
        if (e) {
            e.preventDefault();
        }
        if (page.selected !== undefined) {
            // Si la condition est bonne alors on remonte en haut de la page (a utiliser lorsqu'on clique en bas de la pagination d'une category)
            if (page.name && page.name === 'onPaginationChange') {
                // scrollTo est un Number qui indiquera a combien de pixel du top doit remonter le scroll
                window.scroll({ top: page.scrollTo ? page.scrollTo : 0, behavior: 'smooth' });
            }
            page = page.selected + 1;
        }
        if (typeof window !== 'undefined') {
            window.localStorage.setItem('page', JSON.stringify({ location: this.state.category.name, page }));
        }
        axios.post(`${getAPIUrl()}v2/products/category/${this.state.category._id}`, {
            lang       : this.props.i18n.language,
            withStocks : true,
            PostBody   : {
                structure : {
                    slug              : 1,
                    canonical         : 1,
                    visible           : 1,
                    weight            : 1,
                    pos               : 1,
                    stock             : 1,
                    reviews           : 1,
                    universe          : 1,
                    price             : 1,
                    type              : 1,
                    translation       : 1,
                    images            : 1,
                    pictos            : 1
                },
                limit  : this.state.itemsPerPages,
                filter : filters,
                page   : JSON.parse(window.localStorage.getItem('page')).page,
                sort   : this.state.sort,
            },
        }).then((response) => {
            this.setState({
                current      : page,
                count        : response.data.count,
                productsList : response.data.datas,
                filters      : Object.keys(filters).length !== 0 ? filters : this.state.filters,
            });
        });
    };

    handleSort = ({ field, sortValue }, value, filters = {}) => {
        this.setState({ sort: { [`${field}`]: sortValue }, selectedSort: value }, () => {
            this.onPageChange(1, this.state.itemsPerPages, undefined, filters);
        });
    };

    UNSAFE_componentWillReceiveProps(props) {
        this.setState({
            ...props,
            multilangue   : this.state.multilangue,
            nsLanguages   : this.state.nsLanguages,
            itemsPerPages : defaultItemPerPage,
            selectedSort  : JSON.stringify({ field: 'sortWeight', sortValue: '-1' }),
        });
    }

    render() {
        return (
            <div header={this.state.oCmsHeader.content} footer={this.state.oCmsFooter.content}>
                <div>
                    {this.state.current > 1 && <link rel="prev" href={`${this.state.baseUrl}/${this.state.current - 1}`} />}
                    {this.state.current < Math.ceil(this.state.count / this.state.itemsPerPages) && <link rel="next" href={`${this.state.baseUrl}/${this.state.current + 1}`} />}
                </div>
                <NSBreadcrumb />
                <section className="container catalog">
                    <h2>
                        {this.state.category.name}
                    </h2>
                    <div className="row">
                        <div className="col-md-3">
                            <p>


Filtres
                            </p>
                            <NSFilters min={this.props.min} max={this.props.max} location={category.name} itemspp={this.state.itemsPerPages} reload={(page, pageSize, e, filters) => this.onPageChange(page, pageSize, e, filters)} />
                        </div>
                        <div className="col-md-9 row display-flex">
                            {
                                this.state.productsList.map(product => <NSProductCard url={{ query: this.state.query }} key={product._id} product={product} className="col-md-3 col-sm-4 col-xs-6" />)
                            }
                        </div>
                    </div>
                    <div className="pagination">
                        <Pagination onChange={this.onPageChange} current={parseInt(this.state.current, 0)} pageSize={this.state.itemsPerPages} total={this.state.count} />
                    </div>
                </section>
            </div>
        );
    }

    getChildContext() {
        return {
            getPageState : () => this.state
        };
    }

    static childContextTypes = {
        getPageState : PropTypes.func,
    };

    static defaultProps = {
        contentHtml : {
            content : '',
        },
        category : {
            productsList : [],
            filters      : [],
        },
        oCmsHeader : {
            content : '',
        },
        productsList : [],
        oCmsFooter   : {
            content : '',
        },
    };
}
